# -*- coding: utf-8 -*-
#https://github.com/c0fec0de/anytree
#https://anytree.readthedocs.io/en/2.8.0/
#import anytree
from ..qgisutils import (
    QgisLogger
)
from .anytree import NodeMixin, findall
from .anytree.exporter import DictExporter
class History():
    
    def __init__(self):
        #create a root history node
        self.root_node = HistoryNode(None)
        self.logger = QgisLogger('QgisS3.S3Connection')
        
    def put_result(self, result, contextResult = None):
        resultRef = ResultRef(result)
        self.logger.info(str(resultRef))
        
        historyNode = HistoryNode(resultRef)
        if contextResult:
            parentNode = self.find_historyNode(contextResult)
            if parentNode:
                historyNode.parent = parentNode
                return
        historyNode.parent = self.root_node

        exporter = DictExporter()
        self.logger.info(str(exporter.export(self.root_node)))
        
    
    def find_historyNode(self, result = None):
        if result:
            matches = findall(self.root_node, lambda node: node.represents(result))
            if (len(matches) > 0):
                return matches[0]
        return None
    
    def get(self, filter = None):
        if filter:
            return self.root_node.filter(filter)
        else:
            return self.root_node
    
    def load(self):
        #accept json
        pass
    
    def dump(self):
        #return json
        pass
    
class HistoryNode(NodeMixin):
    def __init__(self, resultRef = None):
        self.resultRef = resultRef
        self.attach_strategy = "prepend"
        
    def clone(self):
        return HistoryNode(self.result)
    
    def represents(self, result):
        if self.resultRef:
            return self.resultRef.equals(result)
        else:
            return False
        
    def filter(self, filter):
        myClone = self.clone()
        for child in self.children:
            childNode = child.filter(filter)
            if childNode:
                childNode.parent = myClone
        if self.is_root or self.matches_filter(filter):
            myClone.in_filter = True
            return myClone
        else:
            if len(myClone.children) > 0:
                return myClone
        
        return None
    
    def matches_filter(self, filter):
        if self.result:
            if (str(self.result['title']).find(filter) > -1) or (self.result['description'] and (self.result['description'].find(filter)) > -1):
                return True
        return False
    
class ResultRef():
    def __init__(self, result):
        self.source = str(result["source"])
        self.typeid = str(result["type"]) 
        self.id = str(result["id"])
        self.title= str(result["title"])
        self.description = None
        if result["description"]:
            self.description = str(result["description"])
            
    def toResult(self):
        return {'source': self.source, 'type': self.typeid ,'id': self.id, 'title': self.title, 'description': self.description}
        
    def equals(self, otherResult):
        return ((self.source == str(otherResult["source"])) and (self.typeid == str(otherResult["type"])) and (self.id == str(otherResult["id"])))
    
    def __str__(self):
        return self.source + "/" + self.typeid + "/" + self.id
 
    