# -*- coding: utf-8 -*-
from builtins import object
from urllib.request import (
    urlopen
)
from qgis.PyQt.QtCore import QCoreApplication, QFileInfo, QUrl, QSettings, QTranslator, qVersion, Qt, QObject, QLocale
import os.path
from qgis.PyQt.QtWidgets import QDockWidget
from .ui import *
from .ui.controller import Controller
from .mysettings import *

class SeptimaSearch(QObject):
    
    def __init__(self, iface):
        super(SeptimaSearch, self).__init__()
        self.iface = iface

        # initialize plugin directory
        self.plugin_dir = os.path.dirname(__file__)
        
        # initialize options
        self.settings = Settings()
        self.settings.settings_updated.connect(self.reset)
        
        self.options_factory = OptionsFactory(self.settings)
        self.options_factory.setTitle(self.tr('OneDoor'))
        iface.registerOptionsWidgetFactory(self.options_factory)
        self.qLocaleToUse = QLocale()
        
        # initialize locale
        locale = QSettings().value('locale/userLocale')
        if QSettings().value('locale/overrideFlag', type=bool):
            self.qLocaleToUse = QLocale(locale)
        
        locale_path = os.path.join(
            self.plugin_dir,
            'i18n',
            '{}.qt.qm'.format(locale[0:2])
        )

        if os.path.exists(locale_path):
            self.translator = QTranslator()
            self.translator.load(locale_path)

            if qVersion() > '4.3.3':
                QCoreApplication.installTranslator(self.translator)

    def reset(self):
        self.controller.connect()
    
    def initGui(self):
        self.controller = Controller(self.settings, self.qLocaleToUse, self.iface)
        response = urlopen("http://septima.dk/qgis-s3/")
        about_html = response.read().decode("utf8")
        self.controller.set_about_html(about_html)

    def unload(self):
        self.iface.unregisterOptionsWidgetFactory(self.options_factory)
        self.controller.unload()


