# -*- coding: utf-8 -*-
#http://s3.demo.septima.dk/organisations/septima/configurations/ejendomsperspektiv/one-door-ng/#/Kortforsyningen/matrikelnumre/2002953-1a/omjordstykket
from qgis.PyQt.QtCore import (
    QObject,
    QTimer,
    QPoint,
    QEvent,
    Qt,
    QUrl
)
from qgis.PyQt.QtWidgets import (
    QApplication,
    QDockWidget
)
from qgis.gui import (
    QgsVertexMarker,
    QgsRubberBand
)
from qgis.core import (
    Qgis,
    QgsCoordinateReferenceSystem,
    QgsCoordinateTransform,
    QgsRectangle,
    QgsGeometry,
    QgsPoint,
    QgsProject,
    QgsWkbTypes
)
from qgis.PyQt.QtGui import (
    QColor,
    QDesktopServices
)

from ..qgisutils import (
    get_current_crs,
    QgisLogger
)

import json
from osgeo import ogr

from ..s3.S3Connection import S3Connection
from ..qgisutils import QgisLogger
import os.path
from .resultlist import ResultList
from .detailview import DetailView
from .historylist import HistoryList
from .leftbar import LeftBar
from .main_widget import MainWidget
class Controller(QObject):

    def __init__(self, settings, qLocaleToUse, qgisIface):
        super(Controller, self).__init__()

        self.iface = qgisIface
        self.settings = settings
        self.qLocaleToUse = qLocaleToUse
        self.cursor_position = None
        self.selectedObject = None
        self.isUnloaded = False
        self.search_target = None
        self.about_html = ""

        self.logger = QgisLogger('QgisS3')
        self.defaultSRID = "epsg:25832"

        self.timer = QTimer(self)
        self.timer.setSingleShot(True)
        self.timer.setInterval(500)
        self.timer.timeout.connect(self.queryS3)

        # Searchbox
        self.MainWidget = MainWidget(self.iface, self.settings)
        self.searchdockwidget = QDockWidget(
            'Septima Search', self.iface.mainWindow()
        )
        
        self.MainWidget.target_clear.connect(self.on_searcbox_target_clear)
        self.MainWidget.text_change.connect(self.queryS3)
        
        self.searchdockwidget.setWindowTitle('OneDoor')
        self.searchdockwidget.setObjectName('SeptimaSearch')
        self.searchdockwidget.setWidget(self.MainWidget)
        self.iface.addDockWidget(
            Qt.RightDockWidgetArea,
            self.searchdockwidget
        )
        
        self.leftbar = LeftBar(self.MainWidget)
        self.leftbar.on_state_selected.connect(self.set_ui_state)
        self.leftbar.on_settings_selected.connect(self.show_settings)
        
        # Initalize Resultlist
        self.resultlist = ResultList(self.MainWidget)
        self.resultlist.on_select.connect(self.on_resultlist_select)
        self.resultlist.on_settings_selected.connect(self.show_settings)
        
        
        self.markers = []
        self.sources = None
        self.set_ui_state('search')
        self.advanced = False

        # Initalize Details View
        self.detailview = DetailView(self.MainWidget, self.qLocaleToUse)
        self.detailview.on_close_clicked.connect(self.show_resultlist)
        self.detailview.on_result_clicked.connect(self.on_detailview_result_clicked)
        self.detailview.on_back_clicked.connect(self.on_detailview_back_clicked)
        self.detailview.on_onedoor_button_clicked.connect(self.on_detailview_onedoor_button_clicked)
        self.detailview.setDefaultSRID(self.defaultSRID)
        
        # Initalize S3Connection
        self.connect()

        # Initalize historylist
        self.historylist = HistoryList(self.MainWidget, self.s3Connection)
        self.historylist.on_select.connect(self.on_historylist_select)

        self.detailview.showAdvanced(False)
        self.leftbar.showAdvanced(False)

    def show_settings(self):
        self.iface.showOptionsDialog(currentPage="onedoorOptions")

    def connect(self):
        connection_info = {
          "host": self.settings.value("host").strip(),
          "token": self.settings.value("token").strip(),
          "organisation": self.settings.value("organisation").strip(),
          "configuration": self.settings.value("configuration").strip()
        }

        self.s3Connection = S3Connection(connection_info)
        # Was:
        #self.s3Connection.connect().then(self.onConnectionSuccess, self.onConnectionError)
        
        self.s3Connection.ping_server().then(self.onPingServerSuccess, self.onConnectionError)

    def onPingServerSuccess(self, ping_result):
        if (len(ping_result["organisations"]) == 0 and len(self.settings.value("token").strip()) == 0):
            text ='<span style="color:red;">Mangler du at angive API Key?</span><br/>Gå til <a href="options">indstillinger->OneDoor</a> for at tilpasse forbindelsesindstillinger'
            self.resultlist.show_error(text)
        else:
            if ("metadata" in ping_result and "defaultSRID" in ping_result["metadata"]):
                self.defaultSRID = ping_result["metadata"]["defaultSRID"]
                self.detailview.setDefaultSRID(self.defaultSRID)
            self.s3Connection.ping_configuration().then(self.onPingConfigurationSuccess, self.onConnectionError)
            
    def onPingConfigurationSuccess(self, ping_result):
        self.s3Connection.connect().then(self.onConnectionSuccess, self.onConnectionError)
        
    def onConnectionError(self, error_message):
        self.logger.critical("S3Connection: Error: " + error_message)
        self.set_ui_state('search')
        self.resultlist.show_error("<pre style='color:red;'>" + error_message + "</pre><br/>Gå til <a href='options'>indstillinger->OneDoor</a> for at tilpasse forbindelsesindstillinger")
        
    def onConnectionSuccess(self):
        try:
            self.sources = self.s3Connection.get_sources()
            self.detailview.set_sources(self.sources)
            
            self.MainWidget.configuration_displayname_text.setText("")

            self.MainWidget.configuration_description_text.setText("")
            
            self.setPluginTitle()
            
            metadata = self.s3Connection.get_metadata()
            if metadata:
                if "displayname" in metadata:
                    self.MainWidget.configuration_displayname_text.setText(metadata["displayname"])
                
                description_html = ""
                if "description" in metadata:
                    description_html = metadata["description"]
                    if "homepage" in metadata:
                        description_html = description_html + " <a href='" + metadata["homepage"]+"'>Læs mere</a>"
                self.MainWidget.configuration_description_text.setOpenExternalLinks(True);
                self.MainWidget.configuration_description_text.setText(description_html)
    
            self.historylist = HistoryList(self.MainWidget, self.s3Connection)
            self.historylist.on_select.connect(self.on_historylist_select)
            
            self.detailview.setS3Connection(self.s3Connection)
            
            self.set_ui_state('search')
            self.do_search(None, None, "")
            
        except Exception as e:
            self.logger.critical("S3Connection error: " + str(e))

    def setPluginTitle(self):
        title = 'OneDoor'
        
        #https://github.com/Septima/qgis-s3/issues/55
        #metadata = self.s3Connection.get_metadata()
        #if metadata:
        #    if "displayname" in metadata:
        #        title = title + " - " + metadata["displayname"]
        #if self.advanced:
        #    config_id  = self.s3Connection.get_config_id()
        #    title = title + " [" + config_id + "]"

        self.searchdockwidget.setWindowTitle(title)
        
    def set_about_html(self, about_html):
        self.about_html = about_html
        self.MainWidget.about_label.setOpenExternalLinks(True);
        self.MainWidget.about_label.setText(about_html);

    def on_detailview_onedoor_button_clicked(self, result, detail):
        host = self.settings.value("host").strip()
        organisation = self.settings.value("organisation").strip()
        configuration = self.settings.value("configuration").strip()
        source = result["source"] 
        type = result["type"]
        id = str(result["id"])
        url = "{}/#/{}/{}/{}/{}/{}".format(host, organisation, configuration, source, type, id)
        if detail:
            url = url + "/" + detail
        QDesktopServices.openUrl(QUrl(url))
        
    def set_ui_state(self, state):
        self.ui_state = state
        self.show_ui_state()
        
    def show_ui_state(self):
        self.MainWidget.details_layout.hide()
        self.clear_marker_geom()
        if self.ui_state == 'search':
            self.MainWidget.resultlist_layout.show()
            self.MainWidget.search_edit_layout.show()
            
            self.MainWidget.history_layout.hide()
            self.MainWidget.historysearch_layout.hide()
            
            self.MainWidget.favorite_layout.hide()
            self.MainWidget.favoritesearch_layout.hide()
            
            self.MainWidget.about_layout.hide()
            return

        if self.ui_state == 'favorites':
            self.MainWidget.resultlist_layout.hide()
            self.MainWidget.search_edit_layout.hide()
            
            self.MainWidget.history_layout.hide()
            self.MainWidget.historysearch_layout.hide()
            
            self.MainWidget.favorite_layout.show()
            self.MainWidget.favoritesearch_layout.show()
            
            self.MainWidget.about_layout.hide()
            return 
    
        if self.ui_state == 'history':
            self.MainWidget.resultlist_layout.hide()
            self.MainWidget.search_edit_layout.hide()
            
            self.MainWidget.favorite_layout.hide()
            self.MainWidget.favoritesearch_layout.hide()

            self.MainWidget.history_layout.show()
            self.MainWidget.historysearch_layout.show()
            self.historylist.show_history()
            
            self.MainWidget.about_layout.hide()
            return
         
        if self.ui_state == 'about':
            self.MainWidget.resultlist_layout.hide()
            self.MainWidget.search_edit_layout.hide()
            
            self.MainWidget.favorite_layout.hide()
            self.MainWidget.favoritesearch_layout.hide()

            self.MainWidget.history_layout.hide()
            self.MainWidget.historysearch_layout.hide()
            
            self.MainWidget.about_layout.show()
            return
        
    def hide_ui_state(self):
            self.MainWidget.resultlist_layout.hide()
            
            self.MainWidget.history_layout.hide()
            self.MainWidget.historysearch_layout.hide()
            
            self.MainWidget.favorite_layout.hide()
            self.MainWidget.favoritesearch_layout.hide()
        
            self.MainWidget.about_layout.hide()

    def show_resultlist(self):
        self.show_ui_state()
        self.clear_marker_geom()
    
    def show_detailview(self, result):
        self.detailview.set_result(result)
        self.hide_ui_state()
        self.MainWidget.details_layout.show()

    def on_searcbox_target_clear(self):
        self.search_target = None
        self.queryS3()

    def on_s3_results(self, results):
        self.show_resultlist()
        self.resultlist.show_results(results)

    def on_detailview_back_clicked(self, result):
        self.show_result_geom(result)
        
    def on_detailview_result_clicked(self, result, contextResult):
        self.s3Connection.get_complete_result(result).then(self.on_s3_result, self.on_s3_error)
        self.s3Connection.put_history(result, contextResult)

    def on_historylist_select(self, selectedObject):
            self.s3Connection.get_complete_result(selectedObject).then(self.on_s3_result, self.on_s3_error)
            self.s3Connection.put_history(selectedObject)
            return
        
    def on_resultlist_select(self, selectedObject):
        self.timer.stop()

        self.MainWidget.setFocus()
        
        if selectedObject:
            self.selectedObject = selectedObject
            
            if selectedObject["item"] == "newquery":
                self.do_search(selectedObject["source"], selectedObject["type"], selectedObject["newquery"])
                return

            if selectedObject["item"] == "shortresult":
                self.s3Connection.get_complete_result(selectedObject).then(self.on_s3_result, self.on_s3_error)
                self.s3Connection.put_history(selectedObject)
                return
                
            if selectedObject["item"] == "deepresult":
                self.on_s3_result(selectedObject)
                return

    def on_s3_result(self, result):
        self.show_result_geom(result)
        self.show_detailview(result)

    def on_s3_error(self, obj):
        self.show_resultlist()
        self.resultlist.show_error(obj)

    def clear_popup(self):
        self.resultlist.clear()

    def do_search(self, source, type, query):
        if source:
            self.search_target = {"source": source, "type": type}

        self.MainWidget.set_text(query.strip())
        self.queryS3()

        if not self.search_target:
            return
        
        display_name = self.get_type_displayname(source, type)
        self.MainWidget.set_target(display_name)
        
    def get_type_displayname(self, source, type):
        if self.sources:
            for known_source in self.sources:
                if known_source["source"] == source:
                    for known_type in known_source["types"]:
                        if known_type["id"] == type:
                            return known_type["singular"]
        return type

    def reset_search_target(self):
        self.search_target = None
        self.queryS3()

    def show_result_geom(self, result):
        self.prevent_suggest()
        if not result:
            return

        # Create a QGIS geom to represent object
        geom = None
        if 'geometry' in result:
            geometry = result['geometry']
            
            #Convert to qGisGeom
            geo_json = json.dumps(geometry)
            ogr_geom = ogr.CreateGeometryFromJson(geo_json)
            wkt = ogr_geom.ExportToWkt()
            qGisGeom = QgsGeometry.fromWkt(wkt)
            
            resultCrs = self.defaultSRID
            
            if ("crs" in geometry and "properties" in geometry["crs"] and "name" in geometry["crs"]["properties"]):
                resultCrs = geometry["crs"]["properties"]["name"]

            srcCrs = QgsCoordinateReferenceSystem(resultCrs)
            dstCrs = get_current_crs(self.iface)
            crsTransform = QgsCoordinateTransform(srcCrs, dstCrs, QgsProject.instance())
            qGisGeom.transform(crsTransform)

            if qGisGeom != None:
                self.set_marker_geom(qGisGeom)
                
                # Zoom to feature
                mc = self.iface.mapCanvas()
                # is isGeographic
                if (dstCrs.isGeographic()):
                    rect = qGisGeom.boundingBox()
                else:
                    bufgeom = qGisGeom.buffer(100.0, 2)
                    rect = bufgeom.boundingBox()
                
                # Finish with refresh
                mc.setExtent(rect)
                mc.refresh()
    
                result['qGisGeom'] = qGisGeom

    def set_marker_geom(self, geom):
        # Show geometry
        self.clear_marker_geom()
        self._set_marker_geom(geom)

    def _set_marker_geom(self, geom):
        if geom.isMultipart():
            geometries = self._extract_single(geom)
            for g in geometries:
                self._set_marker_geom(g)
        else:
            if geom.type() ==  QgsWkbTypes.PointGeometry:
                m = self._set_point_marker(geom)
                m.setColor(QColor(255, 0, 0))
            elif geom.type() in (QgsWkbTypes.LineGeometry, QgsWkbTypes.PolygonGeometry):
                m = self._set_rubberband_marker(geom)
                m.setStrokeColor(QColor(255, 0, 0))
                m.setWidth(3)
            self.markers.append(m)

    def _set_point_marker(self, pointgeom):
        m = QgsVertexMarker(self.iface.mapCanvas())
        m.setIconType(4)
        m.setPenWidth(3)
        m.setCenter(pointgeom.asPoint())
        return m

    def _set_rubberband_marker(self, geom):
        m = QgsRubberBand(self.iface.mapCanvas(), False)  # not polygon
#        if geom.wkbType() == QgsWkbTypes.LineString:
#            linegeom = geom
#        elif geom.wkbType() == QgsWkbTypes.Polygon:
#            linegeom = QgsGeometry.fromPolyline(geom.asPolygon()[0])
#        #m.setToGeometry(linegeom, None)
        m.setToGeometry(geom, None)
        return m

    def clear_marker_geom(self):
        if self.markers:
            for m in self.markers:
                self.iface.mapCanvas().scene().removeItem(m)
        self.markers = []

    def _extract_single(self, geom):
        multiGeom = QgsGeometry()
        geometries = []
        if geom.type() == QgsWkbTypes.PointGeometry:
            if geom.isMultipart():
                multiGeom = geom.asMultiPoint()
                for i in multiGeom:
                    geometries.append(QgsGeometry().fromPointXY(i))
            else:
                geometries.append(geom)
        elif geom.type() == QgsWkbTypes.LineGeometry:
            if geom.isMultipart():
                multiGeom = geom.asMultiPolyline()
                for i in multiGeom:
                    geometries.append(QgsGeometry().fromPolylineXY(i))
            else:
                geometries.append(geom)
        elif geom.type() == QgsWkbTypes.PolygonGeometry:
            if geom.isMultipart():
                multiGeom = geom.asMultiPolygon()
                for i in multiGeom:
                    geometries.append(QgsGeometry().fromPolygonXY(i))
            else:
                geometries.append(geom)
        return geometries

    def prevent_suggest(self):
        self.timer.stop()

    def setAdvanced(self, advanced):
        if advanced:
            self.advanced = True
            self.leftbar.showAdvanced(True)
            self.detailview.showAdvanced(True)
        else:
            self.advanced = False
            self.set_ui_state('search')
            self.detailview.showAdvanced(False)
            self.leftbar.showAdvanced(False)
            
        self.setPluginTitle()
        
    def queryS3(self):
        self.resultlist.clear()
        term = self.MainWidget.text().strip()
        if term == 'set test on':
            self.setAdvanced(True)
        if term == 'set test off':
            self.setAdvanced(False)
        queryResult = None
        if self.search_target:
            queryResult = self.s3Connection.query(self.search_target["source"], self.search_target["type"], term)
        else:
            queryResult = self.s3Connection.query(None, None, term)
        queryResult.then(self.on_s3_results, self.on_s3_error)

    def unload(self):
        self.clear_marker_geom()
        self.MainWidget.unload()
        self.resultlist.unload()
        del self.s3Connection
        self.isUnloaded = True
        self.MainWidget.setParent(None)
        self.searchdockwidget.setParent(None)
        self.iface.removeDockWidget(self.searchdockwidget)
        del self.MainWidget
        del self.searchdockwidget
