# -*- coding: utf-8 -*-
from base64 import b64decode
from urllib.request import urlopen
from qgis.PyQt.QtCore import (
    Qt,
    QSize
)

from qgis.PyQt.QtWidgets import (
    QLabel
)
from qgis.PyQt.QtGui import (
    QPixmap
)
from .widget_factory import WidgetFactory

class DetailItemFactory():
    def __init__(self, container, on_detail_result_clicked, qLocaleToUse):
        self.container = container
        self.qLocaleToUse = qLocaleToUse
        self.widgetFactory = WidgetFactory(self.container, qLocaleToUse)
        self.on_detail_result_clicked = on_detail_result_clicked
        self.levels = {
            0:"",
            1:"margin-left: 10px",
            2:"margin-left: 20px",
            3:"margin-left: 30px"
            }

    def render_item(self, item, grid_layout, row, level):
        rendered_rows = None
        try:
            if item["type"] == "area":
                self.render_area(item, grid_layout, row, level)
            elif item["type"] == "listheader":
                rendered_rows = self.render_listheader(item, grid_layout, row, level)
            elif item["type"] == "textarea":
                self.render_textarea(item, grid_layout, row, level)
            elif item["type"] == "labelvalue":
                rendered_rows = self.render_labelvalue(item, grid_layout, row, level)
            elif item["type"] == "image":
                rendered_rows = self.render_image(item, grid_layout, row, level)
            elif item["type"] == "link":
                self.render_link(item, grid_layout, row, level)
            elif item["type"] == "result":
                self.render_result(item, grid_layout, row, level)
            elif item["type"] == "list":
                rendered_rows = self.render_list(item, grid_layout, row, level)

            if rendered_rows:
                return rendered_rows
            else:
                return 1
        except Exception as e:
            raise Exception ("Exception in DetailItemFactory.render_item: " + str(e))

    def render_list_item(self, item, grid_layout, row, level ):
        rendered_rows = None
        try:
            if item["type"] == "labelvalue":
                rendered_rows = self.render_list_labelvalue(item, grid_layout, row, level)
            elif item["type"] == "result":
                self.render_list_result(item, grid_layout, row, level)
            else:
                return self.render_item(item, grid_layout, row, level)

            if rendered_rows:
                return rendered_rows
            else:
                return 1
        except Exception as e:
            raise Exception ("Exception in DetailItemFactory.render_list_item: " + str(e))

    def render_list(self, list_item, grid_layout, row, level):
        item_type = list_item["itemType"]
        is_homogenous = list_item["isHomogenous"]
        try:
            items = list_item["items"]
            item_count = len(items)
            rendered_rows = 0
            
            rendered_rows = rendered_rows + self.render_listheader(list_item, grid_layout, row, level)

            if item_count > 0:
                for index in range(item_count):
                    if is_homogenous:
                        rendered_rows = rendered_rows + self.render_list_item(items[index], grid_layout, row + rendered_rows, level)
                    else:
                        if items[index]["type"] == 'list':
                            rendered_rows = rendered_rows + self.render_item(items[index], grid_layout, row + rendered_rows, level + 1)
                        else:
                            rendered_rows = rendered_rows + self.render_item(items[index], grid_layout, row + rendered_rows, level)
                #rendered_rows = rendered_rows + self.render_blankrow(grid_layout, row + rendered_rows)
            return rendered_rows
        except Exception as e:
            raise Exception ("Exception in DetailItemFactory.render_list: " + str(e))

    def render_listheader(self, list_item, grid_layout, row, level):
        rendered_rows = 0
        margenleft = self.levels[level]

        has_header = "header" in list_item
        has_firstcolumnheader = False
        if list_item["isHomogenous"] and len(list_item["items"]) > 0 and "label" in list_item["items"][0]:
            has_firstcolumnheader = True
            firstcolumnheader = list_item["items"][0]["label"]
        has_infoitems = "infoItemsHeaders" in list_item

        titleWidget = None
        if has_header:
            titleWidget = QLabel(list_item["header"], self.container)
            titleWidget.setProperty("class", "listheader")
            titleWidget.setStyleSheet("padding-left: 1px; padding-bottom: 3px; padding-top: 3px;" + margenleft)

        headerWidgets = []
        if has_infoitems:
            for index in range(len(list_item["infoItemsHeaders"])):
                infoItemHeader = list_item["infoItemsHeaders"][index]
                labelText = ""
                if "label" in infoItemHeader:
                    labelText = infoItemHeader["label"]
                infoItemLabel = QLabel(labelText)
                infoItemLabel.setProperty("class", "columnheader")
                infoItemLabel.setStyleSheet("padding-left: 1px; padding-bottom: 3px; padding-top: 3px;")
                headerWidgets.append(infoItemLabel)
                
        if has_header:
            if has_infoitems == False:
                grid_layout.addWidget(titleWidget, row, 0, 1, 10)
                rendered_rows = 1
            else:
                if has_firstcolumnheader:
                    grid_layout.addWidget(titleWidget, row, 0, 1, 10)
                    rendered_rows = 1
                    firstColumnLabel = QLabel(firstcolumnheader)
                    firstColumnLabel.setProperty("class", "columnheader")
                    firstColumnLabel.setStyleSheet("padding-left: 1px; padding-bottom: 3px; padding-top: 3px;" + margenleft)
                    grid_layout.addWidget(firstColumnLabel, row + rendered_rows, 0, 1, 1)
                    for index in range(len(headerWidgets)):
                        grid_layout.addWidget(headerWidgets[index], row + rendered_rows, 1 + index, 1, 1)
                    rendered_rows = 2
                else:
                    titleWidget.setProperty("class", "columnheader")
                    grid_layout.addWidget(titleWidget, row + rendered_rows, 0, 1, 1)
                    for index in range(len(headerWidgets)):
                        grid_layout.addWidget(headerWidgets[index], row + rendered_rows, 1 + index, 1, 1)
                    rendered_rows = 1
        else:
            if has_infoitems:
                for index in range(len(headerWidgets)):
                    grid_layout.addWidget(headerWidgets[index], row + rendered_rows, 1 + index, 1, 1)
                rendered_rows = 1
                
        return rendered_rows
    
        try:
            pass
        except Exception as e:
            raise Exception ("Exception in DetailItemFactory.render_listheader: " + str(e))

    def render_blankrow(self, grid_layout, row):
        label = QLabel("")
        #label.setStyleSheet("height: 1px")
        grid_layout.addWidget(label, row, 0)
        return 1
    
    def render_result(self, item, grid_layout, row, level):
        try:
            margenleft = self.levels[level]
            result = item["result"]
            resultWidget = self.widgetFactory.create_result_widget(result)
            resultWidget.clicked.connect(lambda : self.on_detail_result_clicked(result))
            colUsed = 0

            if "label" in item:
                label = self.widgetFactory.create_label(item["label"])
                label.setStyleSheet("padding-left: 1px;" + margenleft)
                if label:
                    grid_layout.addWidget(label, row, colUsed)
                    colUsed = colUsed + 1
                    
            if colUsed == 0:
                resultWidget.setStyleSheet("QPushButton{" + margenleft + "}")
            
            grid_layout.addWidget(resultWidget, row, colUsed)
            colUsed = colUsed + 1

            if "description" in result:
                label = self.widgetFactory.create_label(result["description"])
                label.setStyleSheet("padding-left: 1px;")
                label.setTextInteractionFlags(Qt.TextSelectableByMouse);
                if label:
                    grid_layout.addWidget(label, row, colUsed, 1, 4 - colUsed)
                    colUsed = colUsed + 1

        except Exception as e:
            raise Exception ("Exception in DetailItemFactory.render_result: " + str(e))

    def render_list_result(self, item, grid_layout, row, level):
        try:
            margenleft = self.levels[level]
            colUsed = 0
                    
            result = item["result"]
            resultWidget = self.widgetFactory.create_result_widget(result)
            resultWidget.clicked.connect(lambda : self.on_detail_result_clicked(result))
            resultWidget.setStyleSheet("QPushButton{border: 1px solid #CCCCCC; border-radius: 5px;padding: 4px;text-align:left;" + margenleft + "}")

            grid_layout.addWidget(resultWidget, row, colUsed)
            colUsed = colUsed + 1
            
            if "infoItems" in item:
                for index in range(len(item["infoItems"])):
                    infoItem = item["infoItems"][index]
                    infoItemWidget = self.widgetFactory.createInfoItemWidget(infoItem)
                    grid_layout.addWidget(infoItemWidget, row, colUsed, 1, 1)
                    colUsed = colUsed + 1
            else:
                if "description" in result:
                    label = self.widgetFactory.create_label(result["description"])
                    label.setStyleSheet("padding-left: 1px;")
                    label.setTextInteractionFlags(Qt.TextSelectableByMouse);
                    if label:
                        grid_layout.addWidget(label, row, colUsed, 1, 4 - colUsed)
                        colUsed = colUsed + 1
            if colUsed < 4:
                grid_layout.addWidget(QLabel(""), row, colUsed, 1, 4 - colUsed)
                

        except Exception as e:
            raise Exception ("Exception in DetailItemFactory.render_list_result: " + str(e))

    def render_list_labelvalue(self, item, grid_layout, row, level):
        try:
            margenleft = self.levels[level]
            colUsed = 0
            if "value" in item:
                value_widget = self.widgetFactory.create_value_widget(item)
                value_widget.setStyleSheet("padding-left: 1px;" + margenleft)
                grid_layout.addWidget(value_widget, row, colUsed)
                colUsed = colUsed + 1
                if "infoItems" in item:
                    for index in range(len(item["infoItems"])):
                        infoItem = item["infoItems"][index]
                        infoItemWidget = self.widgetFactory.createInfoItemWidget(infoItem)
                        infoItemWidget.setStyleSheet("padding-left: 1px;")
                        grid_layout.addWidget(infoItemWidget, row, colUsed, 1, 1)
                        colUsed = colUsed + 1
                return 1
            return 0

        except Exception as e:
            raise Exception ("Exception in DetailItemFactory.render_list_labelvalue: " + str(e))
        
    def render_link(self, item, grid_layout, row, level):
        try:
            margenleft = self.levels[level]
            link_widget = self.widgetFactory.create_link_widget(item)
            link_widget.setStyleSheet("padding-left: 1px;" + margenleft)
            grid_layout.addWidget(link_widget, row, 0, 1, 2)
        except Exception as e:
            raise Exception ("Exception in DetailItemFactory.render_link: " + str(e))

    def render_area(self, item, grid_layout, row, level):
        try:
            margenleft = self.levels[level]
            if "icon" in item:
                icon = self.widgetFactory.create_icon_label(item["icon"])
                if icon:
                    grid_layout.addWidget(icon, row, 0)
            label = self.widgetFactory.create_label(item["label"])
            if label:
                grid_layout.addWidget(label, row, 1)
            grid_layout.addWidget(QLabel(item["value"], self.container), row, 2)
        except Exception as e:
            raise Exception ("Exception in DetailItemFactory.render_area: " + str(e))

    def render_textarea(self, item, grid_layout, row, level):
        try:
            margenleft = self.levels[level]
            if "text" in item:
                widget = QLabel(item["text"], self.container)
                widget.setStyleSheet("padding-left: 1px;" + margenleft)
                grid_layout.addWidget(widget, row, 0, 1, 2)
        except Exception as e:
            raise Exception ("Exception in DetailItemFactory.render_textarea: " + str(e))

    def render_image(self, item, grid_layout, row, level):
        try:
            margenleft = self.levels[level]
            rowsUsed = 0
            label = None
            if "label" in item:
                label = self.widgetFactory.create_label(item["label"])
                label.setStyleSheet("padding-left: 1px;" + margenleft)
                grid_layout.addWidget(label, row, 0)
                rowsUsed = 1

            image = self.widgetFactory.create_image_label(item["value"])
            if image:
                image.setStyleSheet("padding-left: 1px;" + margenleft)
                grid_layout.addWidget(image, row + rowsUsed, 0, 1, 3)
                rowsUsed = rowsUsed + 1
            return rowsUsed
        except Exception as e:
            raise Exception ("Exception in DetailItemFactory.render_image: " + str(e))

    def render_labelvalue(self, item, grid_layout, row, level):
        try:
            margenleft = self.levels[level]
            value_widget = None
            if "value" in item:
                value_widget = self.widgetFactory.create_value_widget(item)
                
            label = None
            if "label" in item:
                label = self.widgetFactory.create_label(item["label"])
                label.setStyleSheet("padding-left: 1px;" + margenleft)
                #border: 1px solid red; 

            if label:
                if "value" in item:
                    grid_layout.addWidget(label, row, 0)
                    grid_layout.addWidget(value_widget, row, 1, 1, 2)
                else:
                    grid_layout.addWidget(label, row, 0, 1, 2)
            else:
                if "value" in item:
                    value_widget.setStyleSheet("padding-left: 1px;" + margenleft)
                    grid_layout.addWidget(value_widget, row, 0, 1, 2)
            return 1

        except Exception as e:
            raise Exception ("Exception in DetailItemFactory.render_labelvalue: " + str(e))
        

