# -*- coding: utf-8 -*-
from base64 import b64decode

from qgis.PyQt.QtCore import (
    pyqtSignal,
    QObject,
    QTimer,
    QPoint,
    QEvent,
    Qt
)
from qgis.PyQt.QtWidgets import (
    QTreeWidgetItem,
    QApplication,
    QPushButton,
    QFrame,
    QTreeWidget
)
from qgis.PyQt.QtGui import (
    QPalette,
    QPixmap,
    QIcon,
    QKeyEvent
)

class HistoryList(QObject):
    on_select = pyqtSignal(object)
    
    def __init__(self, container, S3Connection):
        super(HistoryList, self).__init__()
        
        self.S3Connection = S3Connection
        self.historyList = container.historyList
        self.historyList.setColumnCount(2)
        self.historyList.setUniformRowHeights(True)
        self.historyList.setRootIsDecorated(False)
        self.historyList.setEditTriggers(QTreeWidget.NoEditTriggers)
        self.historyList.setSelectionBehavior(QTreeWidget.SelectRows)
        self.historyList.setFrameStyle(QFrame.Box | QFrame.Plain)
        self.historyList.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.historyList.setItemsExpandable(False)
        
        self.historyList.header().hide()
        self.historyList.installEventFilter(self)
        self.historyList.setMouseTracking(True)
        
        #self.treeWidget.setWindowFlags(Qt.Popup)
        self.historyList.setFocusPolicy(Qt.NoFocus)
        self.historyList.itemClicked.connect(self.on_treeWidget_select)
        
        
        self.historysearch = container.historysearch

    def on_treeWidget_select(self):
        item = self.historyList.currentItem()
        selectedObject = item.data(2, Qt.UserRole)
        self.on_select.emit(selectedObject.toResult())
        
    def show_history(self):
        root_node = self.S3Connection.get_history()
        self.historyList.setUpdatesEnabled(False)
        self.historyList.clear()
        for child_node in root_node.children:
            item = QTreeWidgetItem(self.historyList)
            item.setText(0, child_node.resultRef.title)
            if child_node.resultRef.description:
                item.setText(1, child_node.resultRef.description)
            else:
                item.setText(1, child_node.resultRef.typeid)
            item.setData(2, Qt.UserRole, child_node.resultRef)
            self.append_children(child_node, item)
            if len(child_node.children) > 0:
                item = QTreeWidgetItem(self.historyList)
            
        self.historyList.expandAll()
        self.historyList.setCurrentItem(self.historyList.topLevelItem(0))
        self.historyList.resizeColumnToContents(0)
        self.historyList.setUpdatesEnabled(True)

    def append_children(self, parent_node, parent_item):
        for child_node in parent_node.children:
            child_item = QTreeWidgetItem(parent_item)
            child_item.setText(0, child_node.resultRef.title)
            if child_node.resultRef.description:
                child_item.setText(1, child_node.resultRef.description)
            else:
                child_item.setText(1, child_node.resultRef.typeid)
            child_item.setData(2, Qt.UserRole, child_node.resultRef)
            self.append_children(child_node, child_item)


