# -*- coding: utf-8 -*-
import os
from qgis.PyQt.QtGui import (
    QPixmap,
    QIcon
)
from qgis.PyQt.QtCore import (
    QObject,
    pyqtSignal,
    QSize
)

class LeftBar(QObject):
    on_state_selected = pyqtSignal(str)
    on_settings_selected = pyqtSignal()
    on_about_selected = pyqtSignal()
    
    def __init__(self, mainWidget):
        super(LeftBar, self).__init__()
        self.mainWidget = mainWidget

        icon_path = os.path.join(os.path.dirname(__file__), 'images', 'leftbar-search.svg')
        self.leftbar_search_icon = QIcon(icon_path)
        icon_path = os.path.join(os.path.dirname(__file__), 'images', 'leftbar-search-selected.svg')
        self.leftbar_search_selected_icon = QIcon(icon_path)
        
        icon_path = os.path.join(os.path.dirname(__file__), 'images', 'leftbar_info.svg')
        self.leftbar_info_icon = QIcon(icon_path) 
        icon_path = os.path.join(os.path.dirname(__file__), 'images', 'leftbar_info_selected.svg')
        self.leftbar_info_selected_icon = QIcon(icon_path) 

        history_icon_path = os.path.join(os.path.dirname(__file__), 'images', 'history.svg')
        self.mainWidget.lmfHistoryButton.setIcon(QIcon(history_icon_path))
        self.mainWidget.lmfHistoryButton.setIconSize(QSize(24, 24));
        
        favorites_icon_path = os.path.join(os.path.dirname(__file__), 'images', 'star-full.svg')
        self.mainWidget.lmfFavoriteButton.setIcon(QIcon(favorites_icon_path))
        self.mainWidget.lmfFavoriteButton.setIconSize(QSize(24, 24));
        
        settings_icon_path = os.path.join(os.path.dirname(__file__), 'images', 'settings.svg')
        self.mainWidget.lmfSettingsButton.setIcon(QIcon(settings_icon_path))
        self.mainWidget.lmfSettingsButton.setIconSize(QSize(24, 24))


        self.searchButtonContainer = self.mainWidget.lmfSearchButtonContainer
        self.searchButton = self.mainWidget.lmfSearchButton
        self.searchButton.setIcon(self.leftbar_search_selected_icon)
        self.searchButton.setIconSize(QSize(24, 24));
        self.searchButton.clicked.connect(lambda : self.on_state_clicked("search"))
        
        self.aboutButtonContainer = self.mainWidget.lmfAboutButtonContainer
        self.aboutButton = self.mainWidget.lmfAboutButton
        self.aboutButton.setIcon(QIcon(self.leftbar_info_icon))
        self.aboutButton.setIconSize(QSize(24, 24));
        self.aboutButton.clicked.connect(lambda : self.on_state_clicked("about"))
        
        self.favoriteButtonContainer = self.mainWidget.lmfFavoriteButtonContainer
        self.favoriteButton = self.mainWidget.lmfFavoriteButton
        self.favoriteButton.clicked.connect(lambda : self.on_state_clicked("favorites"))

        self.historyButtonContainer = self.mainWidget.lmfHistoryButtonContainer
        self.historyButton = self.mainWidget.lmfHistoryButton
        self.historyButton.clicked.connect(lambda : self.on_state_clicked("history"))

        self.settingsButton = self.mainWidget.lmfSettingsButton
        self.settingsButton.clicked.connect(self.on_settingsButton_clicked)
        #self.settingsButton.hide()
        
        self.selectedStyle = "background-color: rgb(240, 240, 240)"
        self.unSelectedStyle = "background-color: rgb(81, 81, 81)"
        
    def on_state_clicked(self, state):
        self.on_state_selected.emit(state)
        self.show_state(state)
    
    def on_settingsButton_clicked(self):
        self.on_settings_selected.emit()
    
    def showAdvanced(self, boolShow):
        if boolShow:
            self.favoriteButtonContainer.show()
            self.historyButtonContainer.show()
        else:
            self.favoriteButtonContainer.hide()
            self.historyButtonContainer.hide()
        
    def show_state(self, state):
        #unselect all
        self.favoriteButtonContainer.setStyleSheet(self.unSelectedStyle)
        self.historyButtonContainer.setStyleSheet(self.unSelectedStyle)
        self.aboutButtonContainer.setStyleSheet(self.unSelectedStyle)
        self.aboutButton.setIcon(QIcon(self.leftbar_info_icon))

        self.searchButtonContainer.setStyleSheet(self.unSelectedStyle)
        self.searchButton.setIcon(self.leftbar_search_icon)
        
        #and select the correct button
        if state == 'search':
            self.searchButtonContainer.setStyleSheet(self.selectedStyle)
            self.searchButton.setIcon(self.leftbar_search_selected_icon)

        if state == 'favorites':
            self.favoriteButtonContainer.setStyleSheet(self.selectedStyle)
        
        if state == 'history':
            self.historyButtonContainer.setStyleSheet(self.selectedStyle)

        if state == 'about':
            self.aboutButtonContainer.setStyleSheet(self.selectedStyle)
            self.aboutButton.setIcon(QIcon(self.leftbar_info_selected_icon))
            
    def hide(self):
        self.mainWidget.leftMenuFrame.hide()
        
    def show(self):
        self.mainWidget.leftMenuFrame.show()