# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import os
import sys
from PyQt5 import uic
from qgis.PyQt.QtCore import (
    Qt,
    QObject,
    QPoint,
    pyqtSignal,
    QSize
)
from qgis.PyQt.QtWidgets import QFrame, QMessageBox
from ..qgisutils import (
    QgisLogger
)
from qgis.PyQt.QtGui import (
    QPixmap,
    QIcon,
    QPalette
)
FORM_CLASS, _ = uic.loadUiType(
    os.path.join(
        os.path.dirname(__file__),
        'main_widget.ui'
    )
)


#https://doc.qt.io/qt-5/stylesheet-examples.html
class MainWidget(QFrame, FORM_CLASS):

    target_clear = pyqtSignal()
    text_change = pyqtSignal(object)

    def __init__(self, qgisIface, settings):
        #super(SearchBox, self).__init__()
        QFrame.__init__(self, qgisIface.mainWindow())
        self.setupUi(self)
        #self.logger = QgisLogger('SeptimaSearch')

        self.qgisIface = qgisIface
        self.settings = settings
        self.markers = []

        self.setFrameStyle(QFrame.StyledPanel + QFrame.Raised)

        self.targetBit.hide()
        self.targetBit.setProperty("class", "targetBit")
        self.historyBit.setProperty("class", "targetBit")
        self.favoriteBit.setProperty("class", "targetBit")

        icon_path = os.path.join(os.path.dirname(__file__), 'images', 'mIconClearText.svg')
        self.targetBit.setIcon(QIcon(icon_path))
        self.targetBit.clicked.connect(
            self.reset_search_target
        )

        #self.search_edit.cleared.connect(self.clear_marker_geom)
        self.search_edit.textEdited.connect(self.on_text_change)
        #self.search_edit.returnPressed.connect(self.return_pressed)
        self.search_edit.cleared.connect(self.on_text_change)
        self.search_edit.setShowSearchIcon( True )
        self.search_edit.setPlaceholderText( self.tr("Search" ))

        self.adjustSize()
        self.resize(50, self.height())
        self.search_edit.setFocus()
        
        self.logger = QgisLogger('QgisS3.MainWidget')

        #Adjust to UITheme
        back_color = self.search_edit.palette().color(QPalette.Window)
        back_color = back_color.lighter(150)
        red = str(back_color.red())
        green = str(back_color.green())
        blue = str(back_color.blue())
        back_rgb = "rgb(" + red + "," + green + "," + blue + ")"

        text_color = self.search_edit.palette().color(QPalette.WindowText)
        red = str(text_color.red())
        green = str(text_color.green())
        blue = str(text_color.blue())
        text_rgb = "rgb(" + red + "," + green + "," + blue + ")"

        stylesheet = ""\
            "*[class='targetBit'] {background-color: " + back_rgb + "; color: " + text_rgb + "; margin-top: 1px;} "\
            "*[class='listheader'] { background-color: " + back_rgb + ";  color: " + text_rgb + "; border-radius: 2px; border-color: " + text_rgb + "; border-width: 1px; border-style: solid; margin-top: 10px;} "\
            "*[class='columnheader'] { background-color: " + back_rgb + ";  color: " + text_rgb + "; border-radius: 2px; border-color: " + text_rgb + "; border-width: 1px; border-style: solid;} "\
            "*[class='listheader'] { background-color: #CCCCCC;  color: " + text_rgb + "; border-radius: 2px; border-color: #CCCCCC; border-width: 1px; border-style: solid; margin-top: 10px;} "\
            "*[class='columnheader'] { background-color: #CCCCCC;  color: " + text_rgb + "; border-radius: 2px; border-color: #CCCCCC; border-width: 1px; border-style: solid;} "\
            "QToolBox { icon-size: 20px;}" \
            "QToolBox::tab {border-color: " + back_rgb + "; border-width: 2px; border-style: solid; border-radius: 4px; font-weight:500; font-size: 12px;}" \
            "QToolBox::tab:selected {background-color: " + back_rgb + "; }" \
            "*[class='detailTabContent'] { background-color: " + back_rgb + ";} "\
            "*[class='resultButton'] { border: 1px solid #CCCCCC; border-radius: 5px;padding: 4px;text-align:left;}"\
            "*[class='resultButton'] { border: 1px solid " + text_rgb + "; border-radius: 5px;padding: 4px;text-align:left;}"\
            "#result_title {font-weight: bold;}"
        
        self.setStyleSheet(stylesheet)
        
    def getResultList(self):
        return self.resultList
    
    def getResultContainer(self):
        return self.resultlist_layout

    def on_text_change(self):
        self.text_change.emit(self.search_edit.text())
        
    def reset_search_target(self):
        self.targetBit.hide()
        self.target_clear.emit()
        
    def set_target(self, target):
        self.targetBit.setText(' ' + target)
        self.targetBit.show()
        
    def set_text(self, search_text):
        if '<select></select>' in search_text:
            self.cursor_position = search_text.find('<select></select>')
            search_text = search_text.replace('<select></select>', '')
            
        self.search_edit.setText(search_text)
        
    def text(self):
        return self.search_edit.text()

    def dimensions(self):
        bottomPos = self.search_edit.mapToGlobal(
                QPoint(0, self.search_edit.height())
            )
        width = self.search_edit.width()
        return (bottomPos, width)
        
    def set_focus(self):
        self.search_edit.setFocus()
        
    def clear(self):
        #self.clearMarkerGeom()
        self.searchEdit.ui.clear()

    def unload(self):
        pass